#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .incrementalversiondata import IncrementalVersionData
from .semanticversiondata import SemanticVersionData
from .versiondata import VersionData
from .versionmanager import VersionManager


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from enum import Enum, auto
from xpl import Console
from xpl import PlatformType, Environment


#--------------------------------------------------------------------------------
# 스크립트 기능 종류.
#--------------------------------------------------------------------------------
class ScriptFunctionType(Enum):
	#--------------------------------------------------------------------------------
	# 멤버 요소 목록.
	#--------------------------------------------------------------------------------
	NONE = auto() # required.

	# 가상 환경.
	VENV_CREATE = auto() # required.
	VENV_DESTROY = auto() # required.
	VENV_ENABLE = auto() # required.
	VENV_DISABLE = auto() # required.
	VENV_UPDATE = auto() # required.
	VENV_CREATE_FORCE = auto() # optional.
	VENV_UPDATE_FORCE = auto() # optional.

	# 실행.
	EXECUTE_SOURCE = auto() # required.
	EXECUTE_TEST = auto()

	# 빌드.
	BUILD_EXECUTABLE = auto() # required.
	BUILD_ARCHIVE = auto() # required.
	BUILD_MSI = auto()


	# 서비스.
	# Windows (service / nssm)
	# Linux (servicectl)
	# MacOS (brew / launchctl)
	SERVICE_REGISTER = auto()
	SERVICE_UNREGISTER = auto()
	SERVICE_START = auto()
	SERVICE_STOP = auto()
	SERVICE_RESTART = auto()
	SERVICE_STATUS = auto()
	SERVICE_UPDATE_DEPLOY = auto()

	# 로그.
	JOURNAL = auto()

	# 웹서버.
	NGINX_SERVICE_RELOAD = auto()
	NGINX_SERVICE_TEST = auto()


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import os
import sys
import unittest
from unittest import TestLoader, TextTestRunner
from xpl import Console, Path, BaseManager



#--------------------------------------------------------------------------------
# 테스트 매니저. (유닛테스트 기반)
#--------------------------------------------------------------------------------
class TestManager(BaseManager):
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__testLoader: TestLoader
	__textTestRunner: TextTestRunner


	#--------------------------------------------------------------------------------
	# 테스트로더 프로퍼티.
	#--------------------------------------------------------------------------------
	@property
	def TestLoader(thisInstance) -> TestLoader:
		return thisInstance.__testLoader


	#--------------------------------------------------------------------------------
	# 텍스트테스트러너 프로퍼티.
	#--------------------------------------------------------------------------------
	@property
	def TextTestRunner(thisInstance) -> TextTestRunner:
		return thisInstance.__textTestRunner


	#--------------------------------------------------------------------------------
	# 초기화 라이프사이클 메서드.
	#--------------------------------------------------------------------------------
	def __init__(thisInstance) -> None:
		thisInstance.__testLoader: TestLoader = TestLoader()
		thisInstance.__textTestRunner: TextTestRunner = TextTestRunner()


	#--------------------------------------------------------------------------------
	# 실행.
	#--------------------------------------------------------------------------------
	@staticmethod
	def Run() -> None:
		executableFilePath: str = os.path.abspath(sys.modules["__main__"].__file__)
		rootPath: str = Path.GetRootPath(executableFilePath)
		sourcePath: str = os.path.join(rootPath, "src")
		# testPath: str = os.path.join(rootPath, "test")

		# 루트 경로 추가.
		# 루트의 자식인 src와 test를 메인 패키지로 추가하기 위한 설정.
		if rootPath not in sys.path:
			sys.path.append(rootPath)
			Console.Print(f"[TestManager] Add Project Root: {rootPath}")

		# 소스 경로 추가.
		# src 안의 서브패키지들을 메인 패키지 처럼 src. 접근 없이 곧바로 사용하기 위한 설정.
		if sourcePath not in sys.path:
			sys.path.append(sourcePath)
			Console.Print(f"[TestManager] Add Source Packages: {sourcePath}")


		# test 디렉토리 내의 test_ 로 시작하는 모든 스크립트 파일을 기준으로 테스트 스위트 생성.
		# 생성된 테스트 스위트를 실행.
		testManager: TestManager = TestManager()
		suite = testManager.TestLoader.discover(start_dir = "test", pattern = "test_*.py")
		testManager.TextTestRunner.run(suite)




#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .installerprojectdirectorystructure import InstallerProjectDirectoryStructure
from .projectdirectorystructure import ProjectDirectoryStructure


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import os
from xpl import Path, BaseManager
from .packagedata import PackageData


#--------------------------------------------------------------------------------
# 전역 상수 목록.
#--------------------------------------------------------------------------------
FILE_WRITETEXT: str= "wt"
UTF8: str = "utf-8"
LINEFEED: str = "\n"


#--------------------------------------------------------------------------------
# 패키지 매니저.
#--------------------------------------------------------------------------------
class PackageManager(BaseManager):
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__packages: List[PackageData]


	#--------------------------------------------------------------------------------
	# 초기화 라이프사이클 메서드.
	#--------------------------------------------------------------------------------
	def __init__(thisInstance) -> None:
		thisInstance.__packages = list()



	#--------------------------------------------------------------------------------
	# 의존성 파일 생성.
	#--------------------------------------------------------------------------------
	def CreateRequirementsFile(thisInstance, requirementsFilePath: str) -> None:
		if requirementsFilePath:
			raise ValueError(requirementsFilePath) 
		path, name, extension = Path.GetPathNameExtensionFromFileFullPath(requirementsFilePath)
		if not path:
			path = Path.GetCurrentWorkingDirectory()
		if not name:
			raise ValueError(requirementsFilePath) 
		if not extension:
			raise ValueError(requirementsFilePath) 
		if os.path.isfile(requirementsFilePath):
			raise FileExistsError(requirementsFilePath)

		# 상단 내용 작성.	
		textlines = list()
		textlines.append("# Requirements.txt cannot contain Korean characters. (results in cp949 error)")
		textlines.append("# Never use Korean characters.")
		textlines.append("")
		
		# 순회.
		for packageData in thisInstance.__packages:
			packageData = cast(PackageData, packageData)
			textlines.append(f"{packageData.Name}=={packageData.Version}")

		# 저장.
		builtins.print("requirementsFilePath")
		with open(requirementsFilePath, mode = FILE_WRITETEXT, encoding = UTF8) as file:
			content = LINEFEED.join(textlines)
			file.write(content)	


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .basescriptmanager import BaseScriptManager
from .batchscriptmanager import BatchScriptManager
from .scriptfunctiontype import ScriptFunctionType
from .scripttype import ScriptType